import UIKit
import Async
import PlaytubeSDK
import Toast_Swift

protocol DetailsArticlesVCDelegate {
    func refreshArticleData()
}

class DetailsArticlesVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var lblTitle: UILabel!
    @IBOutlet weak var commentTextField: EmojiTextField!
    @IBOutlet weak var emojiButton: UIButton!
    
    // MARK: - Properties
    
    var articlesData: Article?
    var articleCommentArray: [Comment] = []
    var delegate: DetailsArticlesVCDelegate?
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.tabBarController?.tabBar.isHidden = true
        
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        self.tabBarController?.tabBar.isHidden = false
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.delegate?.refreshArticleData()
        self.navigationController?.popViewController(animated: true)
    }
    
    // Copy Button Action
    @IBAction func copyButtonAction(_ sender: UIButton) {
        UIPasteboard.general.string = self.articlesData?.url ?? ""
        self.view.makeToast(NSLocalizedString("Copy to clipboard", comment: "Copy to clipboard"))
    }
    
    // Emoji Button Action
    @IBAction func emojiButtonAction(_ sender: UIButton) {
        self.commentTextField.isEmojiKeyboard = !(self.commentTextField.isEmojiKeyboard)
        self.commentTextField.becomeFirstResponder()
        self.commentTextField.reloadInputViews()
        self.emojiButton.setImage(self.commentTextField.isEmojiKeyboard ? R.image.icon_keyboard_vector() : R.image.emoji_smile(), for: .normal)
    }
    
    // Share Button Action
    @IBAction func shareButtonAction(_ sender: UIButton) {
        self.shareVideo(stringURL: self.articlesData?.url ?? "")
    }
    
    // Send Comment Button Action
    @IBAction func sendCommentButtonAction(_ sender: UIButton) {
        self.sendComment(postID: self.articlesData?.id ?? 0)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.lblTitle.text = articlesData?.title?.htmlAttributedString
        self.commentTextField.delegate = self
        self.registerCell()
        self.fetchArticlesComment()
    }
    
    // Register Cell
    func registerCell() {
        self.tableView.separatorStyle = .none
        tableView.register(UINib(resource: R.nib.articleSectionOneTableItem), forCellReuseIdentifier: R.reuseIdentifier.articleSectionOneTableItem.identifier)
        tableView.register(UINib(resource: R.nib.commentCell), forCellReuseIdentifier: R.reuseIdentifier.commentCell.identifier)
        tableView.register(UINib(resource: R.nib.articlesSectionSixTableItem), forCellReuseIdentifier: R.reuseIdentifier.articlesSectionSixTableItem.identifier)
    }
    
    func shareVideo(stringURL: String) {
        let someText:String = self.articlesData?.url ?? ""
        let objectsToShare: URL = URL(string: stringURL)!
        let sharedObjects: [AnyObject] = [objectsToShare as AnyObject, someText as AnyObject]
        let activityViewController = UIActivityViewController(activityItems : sharedObjects, applicationActivities: nil)
        activityViewController.popoverPresentationController?.sourceView = self.view
        activityViewController.excludedActivityTypes = [
            UIActivity.ActivityType.airDrop,
            UIActivity.ActivityType.postToFacebook,
            UIActivity.ActivityType.postToTwitter,
            UIActivity.ActivityType.mail,
            UIActivity.ActivityType.postToTencentWeibo
        ]
        self.present(activityViewController, animated: true, completion: nil)
    }
    
    func fetchArticlesComment() {
        self.articleCommentArray = []
        // self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
        Async.background {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let postID = self.articlesData?.id ?? 0
            ArticlesManager.instance.getArticlesComments(User_id: userID, Session_Token: sessionID, Limit: 10, Post_id: postID, completionBlock: { (success, sessionError, error) in
                if success != nil {
                    self.dismissProgressDialog {
                        Async.main {
                            self.articleCommentArray = success?.data ?? []
                            self.tableView.reloadData()
                        }
                    }
                } else if sessionError != nil {
                    self.dismissProgressDialog {
                        Async.main {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    }
                } else {
                    self.dismissProgressDialog {
                        Async.main {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                }
            })
        }
    }
    
}

// MARK: - Extensions

// MARK: Api Call
extension DetailsArticlesVC {
    
    func sendComment(postID: Int) {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let commentText = self.commentTextField.text ?? ""
            Async.background {
                ArticlesManager.instance.addArtilcesComments(User_id: userID, Session_Token: sessionID, Post_id: postID, Comment_Text: commentText, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                self.commentTextField.text = ""
                                self.commentTextField.resignFirstResponder()
                                self.fetchArticlesComment()
                                log.verbose("success \(success?.message ?? "") ")
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                                self.view.makeToast(sessionError?.errors?.error_text ?? "")
                            }
                        }
                    } else {
                        Async.main {
                            self.dismissProgressDialog {
                                log.verbose("error = \(error?.localizedDescription ?? "")")
                                self.view.makeToast(error?.localizedDescription ?? "")
                            }
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: TableView Setup
extension DetailsArticlesVC: UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 2
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if section == 1 {
            if self.articleCommentArray.isEmpty {
                return 1
            } else {
                return self.articleCommentArray.count
            }
        } else {
            return 1
        }
    }
        
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        switch indexPath.section {
        case 0:
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.articleSectionOneTableItem.identifier) as! ArticleSectionOneTableItem
            if let article = self.articlesData {
                cell.articleData = article
                cell.bind(article)
                cell.delegate = self
            }
            return cell
        case 1:
            if self.articleCommentArray.isEmpty {
                let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.articlesSectionSixTableItem.identifier) as? ArticlesSectionSixTableItem
                return cell!
            } else {
                let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.commentCell.identifier) as! CommentCell
                cell.delegate = self
                let object = self.articleCommentArray[indexPath.row]
                cell.bind(object)
                return cell
            }
        default:
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.articleSectionOneTableItem.identifier) as? ArticleSectionOneTableItem
            return cell!
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        switch indexPath.section {
        case 1:
            if self.articleCommentArray.isEmpty {
                return 360.0
            } else {
                return UITableView.automaticDimension
            }
        default:
            return UITableView.automaticDimension
        }
    }
    
}

// MARK: ArticleSectionOneTableItemDelegate Methods
extension DetailsArticlesVC: ArticleSectionOneTableItemDelegate {
    
    func handleUserProfileButtonTap(_ sender: UIButton) {
        let newVC = R.storyboard.library.newUserChannelVC()
        newVC?.channelData = self.articlesData?.user_data
        self.navigationController?.pushViewController(newVC!, animated: true)
    }
    
}

// MARK: CommentCellDelegate
extension DetailsArticlesVC: CommentCellDelegate {
    
    func handleRepliesButtonTap(_ sender: UIButton, commentData: Comment?) {
        let newVC = R.storyboard.library.commentReplyVC()
        newVC?.modalPresentationStyle = .custom
        newVC?.transitioningDelegate = self
        newVC?.sheetHeight = UIScreen.main.bounds.height - ((UIScreen.main.bounds.width * 9) / 16)
        newVC?.commentData = commentData
        newVC?.commentType = .ARTICLE
        obj_appDelegate.window?.rootViewController?.present(newVC!, animated: true)
    }
    
}

// MARK: UIViewControllerTransitioningDelegate Methods
extension DetailsArticlesVC: UIViewControllerTransitioningDelegate {
    
    func presentationController(forPresented presented: UIViewController, presenting: UIViewController?, source: UIViewController) -> UIPresentationController? {
        BottomSheetPresentationController(presentedViewController: presented, presenting: presenting)
    }
    
}

// MARK: UITextFieldDelegate Methods
extension DetailsArticlesVC: UITextFieldDelegate {
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        switch textField {
        case self.commentTextField:
            self.commentTextField.isEmojiKeyboard = false
            self.emojiButton.setImage(self.commentTextField.isEmojiKeyboard ? R.image.icon_keyboard_vector() : R.image.emoji_smile(), for: .normal)
        default:
            break
        }
    }
    
}
