import UIKit
import Async
import PlaytubeSDK
import Toast_Swift

protocol PlayerCommentVCDelegate {
    func handleRefreshCommentCount(index:Int?, count: Int?)
}

class PlayerCommentVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var commentTextTextField: UITextField!
    
    // MARK: - Properties
    
    private var isLoading = true {
        didSet {
            tableView.reloadData()
        }
    }
    private var commentsArray: [Comment] = []
    var videoID = 0
    var index = 0
    var delegate: PlayerCommentVCDelegate?
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    @IBAction func sendPressed(_ sender: UIButton) {
        self.sendComment(videoID: self.videoID)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.isLoading = true
        self.setupUI()
        self.fetchComments()
    }
    
    private func setupUI() {
        self.tableView.separatorStyle = .singleLine
        self.tableView.tableFooterView = UIView()
        self.tableView.register(UINib(resource: R.nib.articlesSectionFiveTableItem), forCellReuseIdentifier: R.reuseIdentifier.articlesSectionFiveTableItem.identifier)
        self.tableView.register(UINib(resource: R.nib.articlesSectionSixTableItem), forCellReuseIdentifier: R.reuseIdentifier.articlesSectionSixTableItem.identifier)
    }
    
    func handleRefreshCommentCount() {
        self.delegate?.handleRefreshCommentCount(index: self.index, count: self.commentsArray.count)
    }
    
    func fetchComments() {
        Async.background {
            self.commentsArray = []
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let videoID = self.videoID
            PlayVideoManager.instance.fetchComments(User_id: userID, Session_Token: sessionID, VideoId: videoID, completionBlock: { (success, sessionError, error) in
                if success != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.commentTextTextField.text  = ""
                            self.commentsArray = success?.data ?? []
                            if self.commentsArray.count == 0 {
                                self.tableView.scrollToRow(at: IndexPath(item: 0, section: 0), at: .bottom, animated: false)
                            }
                            self.isLoading = false
                        }
                    }
                } else if sessionError != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.isLoading = false
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text)
                        }
                    }
                } else {
                    Async.main {
                        self.dismissProgressDialog {
                            self.isLoading = false
                            log.error("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription)
                        }
                    }
                }
            })
        }
    }
    
    func sendComment(videoID: Int) {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let commentText = self.commentTextTextField.text ?? ""
            Async.background {
                PlayVideoManager.instance.addComments(User_id: userID, Session_Token: sessionID, VideoId: videoID, Comment_Text: commentText, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                self.commentTextTextField.text  = ""
                                self.commentTextTextField.resignFirstResponder()
                                log.verbose("success \(success?.message ?? "") ")
                                self.fetchComments()
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors.error_text ?? "")
                        }
                    } else {
                        self.dismissProgressDialog {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: TableView Setup
extension PlayerCommentVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if self.isLoading {
            return 10
        } else {
            if self.commentsArray.count == 0 {
                return 1
            } else {
                return self.commentsArray.count
            }
        }
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if isLoading {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.articlesSectionFiveTableItem.identifier) as! ArticlesSectionFiveTableItem
            return cell
        } else {
            if self.commentsArray.count == 0 {
                let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.articlesSectionSixTableItem.identifier) as! ArticlesSectionSixTableItem
                return cell
            } else {
                let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.articlesSectionFiveTableItem.identifier) as! ArticlesSectionFiveTableItem
                cell.playerCommentVC = self
                let object = self.commentsArray[indexPath.row]
                cell.bind(object, articleData: nil)
                return cell
            }
        }
        
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        cell.setTemplateWithSubviews(isLoading, animate: true, viewBackgroundColor: .systemBackground)
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        if self.commentsArray.count == 0 {
            return 360.0
        } else {
            return UITableView.automaticDimension
        }
    }
    
}
