import UIKit
import Async
import PlaytubeSDK
import UIView_Shimmer
import Toast_Swift
import SDWebImage

protocol BlockUserCellDelegate {
    func handleUnblockButtonTap(object: Owner?)
}

class BlockUserCell: UITableViewCell, ShimmeringViewProtocol {
    
    @IBOutlet weak var userName: UILabel!
    @IBOutlet weak var subscriberLabel: UILabel!
    @IBOutlet weak var profileImage: RoundImage!
    @IBOutlet weak var btnSubscribe: UIButton!
    
    var indexPath = IndexPath()
    var object: Owner?
    var shimmeringAnimatedItems: [UIView] {
        [
            userName,
            subscriberLabel,
            profileImage,
            btnSubscribe
        ]
    }
    var delegate: BlockUserCellDelegate?
    
    override func awakeFromNib() {
        super.awakeFromNib()
    }
    
    @IBAction func unblockButton(_ sender: UIButton) {
        self.delegate?.handleUnblockButtonTap(object: self.object)
    }
    
    func bind(_ object: Owner?) {
        self.object = object
        self.userName.text = object?.username ?? ""
        let profileImage = URL(string: object?.avatar ?? "")
        switch object?.subscribe_count {
        case .integer(let value):
            self.subscriberLabel.text = "\(value) Subscribers"
        case .string(let value):
            self.subscriberLabel.text = "\(value) Subscribers"
        default:
            break
        }
        self.profileImage.sd_setShowActivityIndicatorView(true)
        self.profileImage.sd_setIndicatorStyle(.medium)
        self.profileImage.sd_setImage(with: profileImage, placeholderImage:R.image.maxresdefault())
    }
    
}

class BlockedUsersVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var noDataImage: UIImageView!
    @IBOutlet weak var tableView: UITableView!
    
    // MARK: - Properties
    
    private var isLoading = true {
        didSet {
            self.tableView.reloadData()
        }
    }
    private var blockedUsers: GetBlockedUserModel.GetBlockedUserSuccessModel?
    var object: Owner?
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.navigationController?.popViewController(animated: true)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.tableViewSetup()
        isLoading = true
        self.fetchBlockedUsers()
    }
    
    // TableView Setup
    func tableViewSetup() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.tableFooterView = UIView()
        self.tableView.addPullRefresh { [weak self] in
            self?.isLoading = true
            self?.noDataImage.isHidden = true
            self?.fetchBlockedUsers()
        }
    }
    
    private func fetchBlockedUsers() {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                BlockedUserManager.instance.getBlockedUser(User_id: userID, Session_Token: sessionID, type: "get", completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            log.verbose("SUCCESS")
                            self.blockedUsers =  success ?? nil
                            if !(self.blockedUsers?.users?.isEmpty ?? false) {
                                self.noDataImage.isHidden = true
                                // self.noDataLabel.isHidden = true
                            } else {
                                self.noDataImage.isHidden = false
                                // self.noDataLabel.isHidden = false
                            }
                            self.isLoading = false
                            self.tableView.stopPullRefreshEver()
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            self.view.makeToast(sessionError?.errors?.error_text)
                            log.verbose("SessionError = \(sessionError?.errors?.error_text ?? "")")
                        }
                    } else {
                        self.dismissProgressDialog {
                            self.view.makeToast(error?.localizedDescription)
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: - Extensions

// MARK: Api Call
extension BlockedUsersVC {
    
    private func unblockUser() {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let blockId = self.object?.id ?? 0
            Async.background {
                BlockedUserManager.instance.blockUnBlockUser(User_id: userID, Session_Token: sessionID, type: "block", blockId: blockId, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.verbose("SUCCESS")
                                self.fetchBlockedUsers()
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            self.view.makeToast(sessionError?.errors?.error_text)
                            log.verbose("SessionError = \(sessionError?.errors?.error_text ?? "")")
                        }
                    } else {
                        self.dismissProgressDialog {
                            self.view.makeToast(error?.localizedDescription)
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: TableView Setup
extension BlockedUsersVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if self.isLoading {
            return 10
        } else {
            return self.blockedUsers?.users?.count ?? 0
        }
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if self.isLoading {
            let cell = tableView.dequeueReusableCell(withIdentifier: "BlockUserCell") as! BlockUserCell
            return cell
        } else {
            let cell = tableView.dequeueReusableCell(withIdentifier: "BlockUserCell") as! BlockUserCell
            let object = self.blockedUsers?.users![indexPath.row]
            cell.object = object
            cell.bind(object)
            cell.delegate = self
            return cell
        }
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        cell.setTemplateWithSubviews(isLoading, animate: true, viewBackgroundColor: .systemBackground)
    }
    
}

// MARK: BlockUserCellDelegate Methods
extension BlockedUsersVC: BlockUserCellDelegate {
    
    func handleUnblockButtonTap(object: Owner?) {
        self.object = object
        let warningPopupVC = R.storyboard.popups.warningPopupVC()
        warningPopupVC?.delegate = self
        warningPopupVC?.messageText = "Do you want to unblock this user"
        warningPopupVC?.okText = "YES"
        warningPopupVC?.cancelText = "NO"
        self.present(warningPopupVC!, animated: true, completion: nil)
    }
    
}

// MARK: WarningPopupVCDelegate Methods
extension BlockedUsersVC: WarningPopupVCDelegate {
    
    func warningPopupOKButtonPressed(_ sender: UIButton) {
        self.unblockUser()
    }
    
}
