//
//  YoutubePlayerView.swift
//  Playtube
//
//  Created by iMac on 09/06/23.
//  Copyright © 2023 ScriptSun. All rights reserved.
//

import UIKit
import AVKit
import youtube_ios_player_helper

class YoutubePlayerView: UIView {
    
    // MARK: - IBOutlets
    
    @IBOutlet var contentView: UIView!
    @IBOutlet weak var youtubePlayerView: YTPlayerView!
    @IBOutlet weak var elapsedTimeLabel: UILabel!
    @IBOutlet weak var totalTimeLabel: UILabel!
    @IBOutlet weak var playbackSlider: CustomSlider!
    @IBOutlet weak var pausePlayButton: UIButton!
    @IBOutlet weak var skipBackwardButton: UIButton!
    @IBOutlet weak var skipForwardButton: UIButton!
    @IBOutlet weak var backwardButton: UIButton!
    @IBOutlet weak var forwardButton: UIButton!
    @IBOutlet weak var fullScreenModeButton: UIButton!
    
    // MARK: - Properties
    
    var playerVars = [
        "controls": 0,
        "playsinline": 1,
        "autohide": 1,
        "showinfo": 0,
        "modestbranding": 1,
        "iv_load_policy": 3,
        "rel": 0
    ]
    var parentView: VideoPlayerContainerView!
    var isPlaying = false
    fileprivate var isScrubAble = false
    weak var delegate: VideoPlayerViewDelegate?
    var videoPlayerMode: VideoPlayerMode = .expanded
    var timer = Timer()
    
    // MARK: - View Initialize Methods
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        self.commonInit()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        self.commonInit()
    }

    deinit {
        self.cleanUpPlayerForReuse()
    }
    
    // MARK: - Selectors
    // Full Screen Mode Button Action
    @IBAction func fullScreenModeButtonAction(_ sender: UIButton) {
        self.delegate?.handleFullScreenVideoPlayer()
    }
    
    // Play Pause Button Action
    @IBAction func playPauseButtonAction(_ sender: UIButton) {
        if isPlaying {
            self.youtubePlayerView.pauseVideo()
            isPlaying = false
            delegate?.videoPlayStatusChanged(isPlaying: false)
        }else {
            self.youtubePlayerView.playVideo()
            isPlaying = true
            delegate?.videoPlayStatusChanged(isPlaying: true)
        }
        timer.invalidate()
        timer = Timer.scheduledTimer(timeInterval: 3, target: self, selector: #selector(timerAction), userInfo: nil, repeats: true)
        
        let imageName = isPlaying ? "bold_ic_round_pause" : "bold_play_arrow"
        sender.setImage(UIImage(named: imageName), for: .normal)
    }
    
    @IBAction func handleTapGestureAction(_ sender: UIButton) {
        switch videoPlayerMode {
        case .expanded:
            guard isScrubAble else { return }
            handleToggleControls()
            DispatchQueue.main.asyncAfter(deadline: .now() + 3) {
                if self.pausePlayButton.isHidden == false {
                    self.timer.invalidate()
                    self.timer = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(self.timerAction), userInfo: nil, repeats: true)
                }
            }
        case .minimized:
            delegate?.handleMaximizeVideoPlayer()
        }
    }
    
    // Skip Backward Button Action
    @IBAction func skipBackwardButtonAction(_ sender: UIButton) {
        self.youtubePlayerView.currentTime({ float, error in
            if let error = error {
                print(error.localizedDescription)
            } else {
                let newTime = max(0, float - 10.0)
                self.youtubePlayerView.seek(toSeconds: newTime, allowSeekAhead: true)
            }
        })
    }
    
    // Skip Forward Button Action
    @IBAction func skipForwardButtonAction(_ sender: UIButton) {
        self.youtubePlayerView.duration { double, error in
            if let error = error {
                print(error.localizedDescription)
            } else {
                self.youtubePlayerView.currentTime({ float, error in
                    if let error = error {
                        print(error.localizedDescription)
                    } else {
                        let newTime = min(Double(float) + 10.0, double)
                        self.youtubePlayerView.seek(toSeconds: Float(newTime), allowSeekAhead: true)
                    }
                })
            }
        }
    }    
    
    // Backward Button Action
    @IBAction func backwardButtonAction(_ sender: UIButton) {
        self.delegate?.playPreviousVideo()
    }
    
    // Forward Button Action
    @IBAction func forwardButtonAction(_ sender: UIButton) {
        self.delegate?.playNextVideo()
    }
    
    // Playback Slider Action
    @IBAction func playbackSliderAction(_ sender: CustomSlider) {
        self.youtubePlayerView.seek(toSeconds: sender.value, allowSeekAhead: true)
    }
        
    // called every time interval from the timer
    @objc func timerAction() {
        timer.invalidate()
        handleToggleControls()
    }
    
    // MARK: - Helper Functions
    
    @objc func updateSlider() {
        // Get the current time of the video and update the slider
        self.youtubePlayerView.currentTime({ float, error in
            if let error = error {
                print(error.localizedDescription)
            } else {
                self.playbackSlider.value = float
                self.elapsedTimeLabel.text = self.formatDuration(TimeInterval(float))
            }
        })
    }
    
    func isEnableForwardButton(_ enable: Bool) {
        self.forwardButton.isEnabled = enable
    }
    
    func isEnableBackwardButton(_ enable: Bool) {
        self.backwardButton.isEnabled = enable
    }
    
    func cleanUpPlayerForReuse() {
        self.prepareViewForReuse()
        self.youtubePlayerView.removeWebView()
        self.playbackSlider.value = 0
        self.isPlaying = false
        self.isScrubAble = false
        self.timer.invalidate()
    }
    
    fileprivate func prepareViewForReuse() {
        let imageName = "bold_ic_round_pause"
        self.pausePlayButton.setImage(UIImage(named: imageName), for: .normal)
        self.elapsedTimeLabel.text = "00:00:00"
        self.isHidden(true)
    }
    
    fileprivate func handleToggleControls() {
        [pausePlayButton,
         skipBackwardButton,
         backwardButton,
         skipForwardButton,
         forwardButton,
         fullScreenModeButton,
         elapsedTimeLabel,
         totalTimeLabel,
         playbackSlider].forEach { view in
            UIView.animate(withDuration: 0.4, delay: 0) {[weak view] in
                guard let view = view else { return }
                view.alpha = view.alpha == 0 ? 1 : 0
                self.isHidden(view.alpha == 0)
            }
        }
    }
    
    func isHidden(_ hide: Bool) {
        let view = [pausePlayButton,
                    skipBackwardButton,
                    backwardButton,
                    skipForwardButton,
                    forwardButton,
                    fullScreenModeButton,
                    elapsedTimeLabel,
                    totalTimeLabel,
                    playbackSlider]
        view.forEach { view in
            view?.isHidden = hide
        }
    }
    
    fileprivate func setUpViews() {
        self.playbackSlider.setThumbImage(#imageLiteral(resourceName: "thumb").withRenderingMode(.alwaysOriginal), for: .normal)
        self.playbackSlider.setThumbImage(#imageLiteral(resourceName: "thumb").withRenderingMode(.alwaysOriginal), for: .highlighted)
    }
    
    private func commonInit() {
        self.initialConfig()
    }
    
    func initialConfig() {
        Bundle.main.loadNibNamed("YoutubePlayerView", owner: self, options: nil)
        self.contentView.isOpaque = false
        self.contentView.backgroundColor = .clear
        self.contentView.frame = self.bounds
        self.addSubview(self.contentView)
        self.setUpViews()
        self.youtubePlayerView.delegate = self
    }
    
    func initializeYoutubePlayer(for video_id: String) {
        self.cleanUpPlayerForReuse()
        if self.youtubePlayerView.load(withVideoId: video_id, playerVars: playerVars) {
            print("Loaded Video")
        } else {
            print("Not Loaded Video")
        }
    }
    
    func formatDuration(_ duration: TimeInterval) -> String {
        let minutes = Int(duration) / 60
        let seconds = Int(duration) % 60
        let hours = minutes / 60
        return String(format: "%02d:%02d:%02d", hours, minutes, seconds)
    }

}

// MARK: - Extensions
extension YoutubePlayerView: YTPlayerViewDelegate {
    
    func playerView(_ playerView: YTPlayerView, receivedError error: YTPlayerError) {
        print(error)
    }
    
    func playerView(_ playerView: YTPlayerView, didChangeTo quality: YTPlaybackQuality) {
        print("Quality >>>>> ", quality.rawValue)
    }
    
    func playerView(_ playerView: YTPlayerView, didPlayTime playTime: Float) {
        self.updateSlider()
    }

    func playerViewDidBecomeReady(_ playerView: YTPlayerView) {
        print(#function)
        self.isScrubAble = true
        self.parentView.activityIndicator.stopAnimating()
        self.playPauseButtonAction(self.pausePlayButton)
        self.youtubePlayerView.duration({ double, error in
            if let error = error {
                print(error.localizedDescription)
            } else {
                self.totalTimeLabel.text = self.formatDuration(double)
                self.playbackSlider.maximumValue = Float(double)
                print("Duration >>>>> ", double)
            }
        })
    }
    
    func playerView(_ playerView: YTPlayerView, didChangeTo state: YTPlayerState) {
        switch state {
        case .ended:
            self.delegate?.autoPlayNextVideo()
            print("enddedd")
            break
        case .unstarted:
            print("")
        case .playing:
            print("")
        case .paused:
            print("")
        case .buffering:
            print("")
        case .cued:
            print("")
        case .unknown:
            print("")
        @unknown default:
            break
        }
    }
    
}
